# -*- coding: utf-8 -*-

import re

class OpeningHoursParser:
	"""
	OpeningHoursParser handles parsing of opening_hours=* tag.
	It only supports basic version of the tag, as we don't need full implementation for public transport hours.
	"""
	def __init__(self, value):
		"""
		Creates the OpeningHoursParser Object with OSM opening_hours string
		# param value (string): The opening_hours=* value
		"""
		self.openingHours = {}
		self._parse(value)

		# Check if opening hours is empty (invalid parsing)
		if len([ k for k in self.openingHours if len(k) == 0 ]) == len(self.openingHours):
			raise Exception("Can't parse opening_hours : "+value)

	def getTable(self):
		"""
		Get the parsed value as a table
		# return (dict): The hours, as { day: [ hours ] }
		"""
		return self.openingHours

	def _parse(self, inp):
		"""
		Parses the input and creates openingHours Object
		"""
		self._initOpeningHoursObj()
		inp = self._simplify(inp)
		parts = self._splitHard(inp)

		for part in parts:
			self._parseHardPart(part)

	def _simplify(self, txt):
		if txt == "24/7":
			txt = "mo-su 00:00-24:00; ph 00:00-24:00"
		txt = txt.lower()
		txt = txt.strip()

		txt = re.sub(" +(?= )", "", txt)

		txt = txt.replace(' -', '-')
		txt = txt.replace('- ', '-')

		txt = txt.replace(' :', ':')
		txt = txt.replace(': ', ':')

		txt = txt.replace(' ,', ',')
		txt = txt.replace(', ', ',')

		txt = txt.replace(' ;', ';')
		txt = txt.replace('; ', ';')
		return txt

	def _splitHard(self, inp):
		return inp.split(';')

	def _parseHardPart(self, part):
		if part == "24/7":
			part = "mo-su 00:00-24:00"

		segments = re.compile("\ |\,").split(part)

		tempData = {}
		days = []
		times = []
		isValid = False

		for i in range(len(segments)):
			segment = segments[i]
			if self._checkDay(segment):
				isValid = True
				if len(times) == 0:
					days = days + self._parseDays(segment)
				else:
					# append
					for day in days:
						if tempData[day]:
							tempData[day] = tempData[day] + times
						else:
							tempData[day] = times
					days = self._parseDays(segment)
					times = []
			if self._checkTime(segment):
				isValid = True
				if i == 0 and len(days) == 0:
					days = self._parseDays("Mo-Su,PH")
				if segment == "off":
					times = "off"
				else:
					times.append(self._cleanTime(segment))

		# Raise exception if no read part is valid
		if not isValid:
			raise Exception("Can't parse opening_hours : "+part)

		# commit last times to it days
		for day in days:
			if day in tempData:
				tempData[day] = tempData[day] + times
			else:
				tempData[day] = times

		for day in days:
			if times == "off":
				tempData[day] = []
			elif len(times) == 0:
				tempData[day] = ["00:00-24:00"]

		# apply data to main obj
		for key in tempData:
			self.openingHours[key] = tempData[key]

	def _parseDays(self, part):
		part = part.lower()
		days = []
		softparts = part.split(',')
		for part in softparts:
			rangecount = len(re.findall("\-", part))
			if rangecount == 0:
				days.append(part)
			else:
				days = days + self._calcDayRange(part)

		return days

	def _cleanTime(self, time):
		if re.match("^[0-9]:[0-9]{2}", time):
			time = "0"+time
		if re.match("^[0-9]{2}:[0-9]{2}\-[0-9]:[0-9]{2}", time):
			time = time[0:6]+"0"+time[6:]
		return time

	def _initOpeningHoursObj(self):
		self.openingHours = {
			"su": [],
			"mo": [],
			"tu": [],
			"we": [],
			"th": [],
			"fr": [],
			"sa": [],
			"ph": []
		}

	def _calcDayRange(self, daysRange):
		"""
		Calculates the days in range "mo-we" -> ["mo", "tu", "we"]
		"""
		defDays = {
			"su": 0,
			"mo": 1,
			"tu": 2,
			"we": 3,
			"th": 4,
			"fr": 5,
			"sa": 6
		}

		rangeElements = daysRange.split('-')

		dayStart = defDays[rangeElements[0]]
		dayEnd = defDays[rangeElements[1]]

		numberRange = self._calcRange(dayStart, dayEnd, 6)
		outRange = []

		for n in numberRange:
			for key in defDays:
				if defDays[key] == n:
					outRange.append(key)

		return outRange

	def _calcRange(self, minv, maxv, maxval):
		"""
		Creates a range between two number.
		if the max value is 6 a range bewteen 6 and 2 is 6, 0, 1, 2
		"""
		if minv == maxv:
			return [minv]
		myRange = [minv]
		rangepoint = minv
		while rangepoint < (maxv if (minv < maxv) else maxval):
			rangepoint += 1
			myRange.append(rangepoint)
		if minv > maxv:
			# add from first in list to max value
			myRange = myRange + self._calcRange(0, maxv, maxval)

		return myRange

	def _checkTime(self, inp):
		"""
		Check if string is time range
		"""
		# e.g. 09:00+
		if re.search("[0-9]{1,2}:[0-9]{2}\+", inp):
			return True
		# e.g. 08:00-12:00
		if re.search("[0-9]{1,2}:[0-9]{2}\-[0-9]{1,2}:[0-9]{2}", inp):
			return True
		# off
		if re.search("off", inp):
			return True
		return False

	def _checkDay(self, inp):
		"""
		check if string is day or dayrange
		"""
		days = ["mo", "tu", "we", "th", "fr", "sa", "su", "ph"]
		if re.search("\-", inp):
			rangelements = inp.split('-')
			if rangelements[0] in days and rangelements[1] in days:
				return True
		else:
			if inp in days:
				return True
		return False



# -*- coding: utf-8 -*-

from transporthours.openinghoursparser import OpeningHoursParser
import unittest

class OpeningHoursParserTest(unittest.TestCase):
	#
	# Constructor
	#
	def test_constructor_handles_hours(self):
		value = "10:00-12:00"
		oh = OpeningHoursParser(value)
		result = oh.getTable()
		expected = {
			"mo": ["10:00-12:00"],
			"tu": ["10:00-12:00"],
			"we": ["10:00-12:00"],
			"th": ["10:00-12:00"],
			"fr": ["10:00-12:00"],
			"sa": ["10:00-12:00"],
			"su": ["10:00-12:00"],
			"ph": ["10:00-12:00"]
		}
		self.assertEqual(result, expected)

	def test_constructor_handles_ph_off(self):
		value = "10:00-12:00; PH off";
		oh = OpeningHoursParser(value)
		result = oh.getTable()
		expected = {
			"mo": ["10:00-12:00"],
			"tu": ["10:00-12:00"],
			"we": ["10:00-12:00"],
			"th": ["10:00-12:00"],
			"fr": ["10:00-12:00"],
			"sa": ["10:00-12:00"],
			"su": ["10:00-12:00"],
			"ph": []
		}
		self.assertEqual(result, expected)

	def test_constructor_handles_full_week_ph(self):
		value = "Mo-Su,PH 10:00-12:00"
		oh = OpeningHoursParser(value)
		result = oh.getTable()
		expected = {
			"mo": ["10:00-12:00"],
			"tu": ["10:00-12:00"],
			"we": ["10:00-12:00"],
			"th": ["10:00-12:00"],
			"fr": ["10:00-12:00"],
			"sa": ["10:00-12:00"],
			"su": ["10:00-12:00"],
			"ph": ["10:00-12:00"]
		}
		self.assertEqual(result, expected)

	def test_constructor_handles_day_hours(self):
		value = "Mo 10:00-12:00"
		oh = OpeningHoursParser(value)
		result = oh.getTable()
		expected = {
			"mo": ["10:00-12:00"],
			"tu": [],
			"we": [],
			"th": [],
			"fr": [],
			"sa": [],
			"su": [],
			"ph": []
		}
		self.assertEqual(result, expected)

	def test_constructor_handles_day_hours_short(self):
		value = "Mo 1:00-5:00"
		oh = OpeningHoursParser(value)
		result = oh.getTable()
		expected = {
			"mo": ["01:00-05:00"],
			"tu": [],
			"we": [],
			"th": [],
			"fr": [],
			"sa": [],
			"su": [],
			"ph": []
		}
		self.assertEqual(result, expected)

	def test_constructor_handles_days_ranges_hours(self):
		value = "Mo-We 10:00-12:00"
		oh = OpeningHoursParser(value)
		result = oh.getTable()
		expected = {
			"mo": ["10:00-12:00"],
			"tu": ["10:00-12:00"],
			"we": ["10:00-12:00"],
			"th": [],
			"fr": [],
			"sa": [],
			"su": [],
			"ph": []
		}
		self.assertEqual(result, expected)

	def test_constructor_handles_days_list_hours(self):
		value = "Mo,We 10:00-12:00"
		oh = OpeningHoursParser(value)
		result = oh.getTable()
		expected = {
			"mo": ["10:00-12:00"],
			"tu": [],
			"we": ["10:00-12:00"],
			"th": [],
			"fr": [],
			"sa": [],
			"su": [],
			"ph": []
		}
		self.assertEqual(result, expected)

	def test_constructor_handles_days_different_hours(self):
		value = "Mo-We 10:00-12:00; Th-Sa 09:00-15:00"
		oh = OpeningHoursParser(value)
		result = oh.getTable()
		expected = {
			"mo": ["10:00-12:00"],
			"tu": ["10:00-12:00"],
			"we": ["10:00-12:00"],
			"th": ["09:00-15:00"],
			"fr": ["09:00-15:00"],
			"sa": ["09:00-15:00"],
			"su": [],
			"ph": []
		}
		self.assertEqual(result, expected)

	def test_constructor_handles_hours_ranges_same_day(self):
		value = "Mo-We 10:00-12:00, 17:00-19:00"
		oh = OpeningHoursParser(value)
		result = oh.getTable()
		expected = {
			"mo": ["10:00-12:00", "17:00-19:00"],
			"tu": ["10:00-12:00", "17:00-19:00"],
			"we": ["10:00-12:00", "17:00-19:00"],
			"th": [],
			"fr": [],
			"sa": [],
			"su": [],
			"ph": []
		}
		self.assertEqual(result, expected)

	def test_constructor_handles_days_only(self):
		value = "Mo-We"
		oh = OpeningHoursParser(value)
		result = oh.getTable()
		expected = {
			"mo": ["00:00-24:00"],
			"tu": ["00:00-24:00"],
			"we": ["00:00-24:00"],
			"th": [],
			"fr": [],
			"sa": [],
			"su": [],
			"ph": []
		}
		self.assertEqual(result, expected)

	def test_constructor_handles_days_only_subpart(self):
		value = "Mo-We; Sa 01:00-15:00"
		oh = OpeningHoursParser(value)
		result = oh.getTable()
		expected = {
			"mo": ["00:00-24:00"],
			"tu": ["00:00-24:00"],
			"we": ["00:00-24:00"],
			"th": [],
			"fr": [],
			"sa": ["01:00-15:00"],
			"su": [],
			"ph": []
		}
		self.assertEqual(result, expected)

	def test_constructor_handles_invalid(self):
		value = "This is not opening_hours !"
		self.assertRaises(Exception, OpeningHoursParser, value)

if __name__ == '__main__':
	unittest.main()



# -*- coding: utf-8 -*-

from transporthours.main import Main
from collections import OrderedDict
import unittest

class MainTest(unittest.TestCase):
	#
	# tagsToGtfs
	#
	def test_tagsToGtfs_works_singlerange(self):
		tags = {
			"type": "route",
			"route": "bus",
			"name": "Ligne 42",
			"opening_hours": "Mo-Fr 05:00-22:00",
			"interval": "00:30",
			"interval:conditional": "00:10 @ (Mo-Fr 07:00-09:30, 16:30-19:00)"
		}
		result = Main().tagsToGtfs(tags)
		expected = [
			{ "monday": True, "tuesday": True, "wednesday": True, "thursday": True, "friday": True, "saturday": False, "sunday": False, "start_time": "05:00:00", "end_time": "07:00:00", "headway": 1800 },
			{ "monday": True, "tuesday": True, "wednesday": True, "thursday": True, "friday": True, "saturday": False, "sunday": False, "start_time": "07:00:00", "end_time": "09:30:00", "headway": 600 },
			{ "monday": True, "tuesday": True, "wednesday": True, "thursday": True, "friday": True, "saturday": False, "sunday": False, "start_time": "09:30:00", "end_time": "16:30:00", "headway": 1800 },
			{ "monday": True, "tuesday": True, "wednesday": True, "thursday": True, "friday": True, "saturday": False, "sunday": False, "start_time": "16:30:00", "end_time": "19:00:00", "headway": 600 },
			{ "monday": True, "tuesday": True, "wednesday": True, "thursday": True, "friday": True, "saturday": False, "sunday": False, "start_time": "19:00:00", "end_time": "22:00:00", "headway": 1800 }
		]
		self.assertEqual(result, expected)

	def test_tagsToGtfs_works_multiranges(self):
		tags = {
			"type": "route",
			"route": "bus",
			"name": "Ligne 42",
			"opening_hours": "Mo-Fr 05:00-22:00; Sa 07:00-23:00",
			"interval": "00:10",
			"interval:conditional": "5 @ (Mo,Fr 08:00-10:00,16:30-18:30); 00:06 @ (Sa 11:00-13:00)"
		}
		result = Main().tagsToGtfs(tags)
		expected = [
			{ "monday": True, "tuesday": False, "wednesday": False, "thursday": False, "friday": True, "saturday": False, "sunday": False, "start_time": "05:00:00", "end_time": "08:00:00", "headway": 600 },
			{ "monday": True, "tuesday": False, "wednesday": False, "thursday": False, "friday": True, "saturday": False, "sunday": False, "start_time": "08:00:00", "end_time": "10:00:00", "headway": 300 },
			{ "monday": True, "tuesday": False, "wednesday": False, "thursday": False, "friday": True, "saturday": False, "sunday": False, "start_time": "10:00:00", "end_time": "16:30:00", "headway": 600 },
			{ "monday": True, "tuesday": False, "wednesday": False, "thursday": False, "friday": True, "saturday": False, "sunday": False, "start_time": "16:30:00", "end_time": "18:30:00", "headway": 300 },
			{ "monday": True, "tuesday": False, "wednesday": False, "thursday": False, "friday": True, "saturday": False, "sunday": False, "start_time": "18:30:00", "end_time": "22:00:00", "headway": 600 },
			{ "monday": False, "tuesday": True, "wednesday": True, "thursday": True, "friday": False, "saturday": False, "sunday": False, "start_time": "05:00:00", "end_time": "22:00:00", "headway": 600 },
			{ "monday": False, "tuesday": False, "wednesday": False, "thursday": False, "friday": False, "saturday": True, "sunday": False, "start_time": "07:00:00", "end_time": "11:00:00", "headway": 600 },
			{ "monday": False, "tuesday": False, "wednesday": False, "thursday": False, "friday": False, "saturday": True, "sunday": False, "start_time": "11:00:00", "end_time": "13:00:00", "headway": 360 },
			{ "monday": False, "tuesday": False, "wednesday": False, "thursday": False, "friday": False, "saturday": True, "sunday": False, "start_time": "13:00:00", "end_time": "23:00:00", "headway": 600 }
		]
		self.assertEqual(result, expected)

	def test_tagsToGtfs_ignore_ph(self):
		tags = {
			"type": "route",
			"route": "bus",
			"name": "Ligne 42",
			"opening_hours": "Mo-Fr,PH 05:00-22:00",
			"interval": "00:30",
			"interval:conditional": "00:10 @ (Mo-Fr 07:00-09:30, 16:30-19:00); 00:30 @ (PH 05:00-22:00)"
		}
		result = Main().tagsToGtfs(tags)
		expected = [
			{ "monday": True, "tuesday": True, "wednesday": True, "thursday": True, "friday": True, "saturday": False, "sunday": False, "start_time": "05:00:00", "end_time": "07:00:00", "headway": 1800 },
			{ "monday": True, "tuesday": True, "wednesday": True, "thursday": True, "friday": True, "saturday": False, "sunday": False, "start_time": "07:00:00", "end_time": "09:30:00", "headway": 600 },
			{ "monday": True, "tuesday": True, "wednesday": True, "thursday": True, "friday": True, "saturday": False, "sunday": False, "start_time": "09:30:00", "end_time": "16:30:00", "headway": 1800 },
			{ "monday": True, "tuesday": True, "wednesday": True, "thursday": True, "friday": True, "saturday": False, "sunday": False, "start_time": "16:30:00", "end_time": "19:00:00", "headway": 600 },
			{ "monday": True, "tuesday": True, "wednesday": True, "thursday": True, "friday": True, "saturday": False, "sunday": False, "start_time": "19:00:00", "end_time": "22:00:00", "headway": 1800 }
		]
		self.assertEqual(result, expected)

	def test_tagsToGtfs_empty_if_unset(self):
		tags = {
			"type": "route",
			"route": "bus",
			"name": "Ligne 42"
		}
		result = Main().tagsToGtfs(tags)
		expected = []
		self.assertEqual(result, expected)

	def test_tagsToGtfs_error_if_invalid(self):
		tags = {
			"type": "route",
			"route": "bus",
			"name": "Ligne 42",
			"opening_hours": "this is broken",
			"interval": "00:10"
		}
		self.assertRaises(Exception, Main().tagsToGtfs, tags)

	#
	# tagsToHoursObject
	#
	def test_tagsToHoursObject_handles_all_tags_set(self):
		tags = {
			"type": "route",
			"route": "bus",
			"name": "Ligne 42",
			"opening_hours": "Mo-Fr 05:00-22:00",
			"interval": "00:30",
			"interval:conditional": "00:10 @ (Mo-Fr 07:00-09:30, 16:30-19:00)"
		}
		result = Main().tagsToHoursObject(tags)
		expected = {
			"opens": {
				"mo": ["05:00-22:00"],
				"tu": ["05:00-22:00"],
				"we": ["05:00-22:00"],
				"th": ["05:00-22:00"],
				"fr": ["05:00-22:00"],
				"sa": [],
				"su": [],
				"ph": []
			},
			"defaultInterval": 30,
			"otherIntervals": [
				{
					"interval": 10,
					"applies": {
						"mo": ["07:00-09:30", "16:30-19:00"],
						"tu": ["07:00-09:30", "16:30-19:00"],
						"we": ["07:00-09:30", "16:30-19:00"],
						"th": ["07:00-09:30", "16:30-19:00"],
						"fr": ["07:00-09:30", "16:30-19:00"],
						"sa": [],
						"su": [],
						"ph": []
					}
				}
			],
			"otherIntervalsByDays": [
				{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "07:00-09:30": 10, "16:30-19:00": 10 } }
			],
			"allComputedIntervals": [
				{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "05:00-07:00": 30, "07:00-09:30": 10, "09:30-16:30": 30, "16:30-19:00": 10, "19:00-22:00": 30 } }
			]
		}
		self.assertEqual(result, expected)

	def test_tagsToHoursObject_handles_no_intervalconditional_tag(self):
		tags = {
			"type": "route",
			"route": "bus",
			"name": "Ligne 42",
			"opening_hours": "Mo-Fr 05:00-22:00",
			"interval": "00:30"
		}
		result = Main().tagsToHoursObject(tags)
		expected = {
			"opens": {
				"mo": ["05:00-22:00"],
				"tu": ["05:00-22:00"],
				"we": ["05:00-22:00"],
				"th": ["05:00-22:00"],
				"fr": ["05:00-22:00"],
				"sa": [],
				"su": [],
				"ph": []
			},
			"defaultInterval": 30,
			"otherIntervals": "unset",
			"otherIntervalsByDays": "unset",
			"allComputedIntervals": [
				{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "05:00-22:00": 30 } }
			]
		}
		self.assertEqual(result, expected)

	def test_tagsToHoursObject_handles_no_opening_hours_tag(self):
		tags = {
			"type": "route",
			"route": "bus",
			"name": "Ligne 42",
			"interval": "00:30"
		}
		result = Main().tagsToHoursObject(tags)
		expected = {
			"opens": "unset",
			"defaultInterval": 30,
			"otherIntervals": "unset",
			"otherIntervalsByDays": "unset",
			"allComputedIntervals": [
				{ "days": [ "mo", "tu", "we", "th", "fr", "sa", "su", "ph" ], "intervals": { "00:00-24:00": 30 } }
			]
		}
		self.assertEqual(result, expected)

	def test_tagsToHoursObject_handles_no_tags(self):
		tags = {
			"type": "route",
			"route": "bus",
			"name": "Ligne 42"
		}
		result = Main().tagsToHoursObject(tags)
		expected = {
			"opens": "unset",
			"defaultInterval": "unset",
			"otherIntervals": "unset",
			"otherIntervalsByDays": "unset",
			"allComputedIntervals": "unset"
		}
		self.assertEqual(result, expected)

	def test_tagsToHoursObject_handles_opening_hours_invalid(self):
		tags = {
			"type": "route",
			"route": "bus",
			"name": "Ligne 42",
			"opening_hours": "what ?",
			"interval": "00:30"
		}
		result = Main().tagsToHoursObject(tags)
		expected = {
			"opens": "invalid",
			"defaultInterval": 30,
			"otherIntervals": "unset",
			"otherIntervalsByDays": "unset",
			"allComputedIntervals": "invalid"
		}
		self.assertEqual(result, expected)

	def test_tagsToHoursObject_handles_interval_invalid(self):
		tags = {
			"type": "route",
			"route": "bus",
			"name": "Ligne 42",
			"interval": "12 minutes is so long to wait for a bus..."
		}
		result = Main().tagsToHoursObject(tags)
		expected = {
			"opens": "unset",
			"defaultInterval": "invalid",
			"otherIntervals": "unset",
			"otherIntervalsByDays": "unset",
			"allComputedIntervals": "invalid"
		}
		self.assertEqual(result, expected)

	def test_tagsToHoursObject_handles_intervalconditional_invalid(self):
		tags = {
			"type": "route",
			"route": "bus",
			"name": "Ligne 42",
			"interval": "00:30",
			"interval:conditional": "12 @ random hours"
		}
		result = Main().tagsToHoursObject(tags)
		expected = {
			"opens": "unset",
			"defaultInterval": 30,
			"otherIntervals": "invalid",
			"otherIntervalsByDays": "invalid",
			"allComputedIntervals": "invalid"
		}
		self.assertEqual(result, expected)

	def test_handles_demo_example(self):
		tags = {
			"interval": "30",
			"interval:conditional": "60 @ (Mo-Fr 20:00-22:00; Sa 05:00-10:00,20:00-22:00; Su 10:00-15:00,18:00-22:00); 15 @ (Mo-Fr 05:00-09:00,16:00-20:00; Su 15:00-18:00)",
			"opening_hours": "Mo-Sa 05:00-22:00; Su 07:00-22:00"
		}
		result = Main().tagsToHoursObject(tags)
		expected = {
			"opens": {
				"mo": ["05:00-22:00"],
				"tu": ["05:00-22:00"],
				"we": ["05:00-22:00"],
				"th": ["05:00-22:00"],
				"fr": ["05:00-22:00"],
				"sa": ["05:00-22:00"],
				"su": ["07:00-22:00"],
				"ph": []
			},
			"defaultInterval": 30,
			"otherIntervals": [
				{
					"interval": 60,
					"applies": {
						"mo": ["20:00-22:00"],
						"tu": ["20:00-22:00"],
						"we": ["20:00-22:00"],
						"th": ["20:00-22:00"],
						"fr": ["20:00-22:00"],
						"sa": ["05:00-10:00","20:00-22:00"],
						"su": ["10:00-15:00","18:00-22:00"],
						"ph": []
					}
				},
				{
					"interval": 15,
					"applies": {
						"mo": ["05:00-09:00","16:00-20:00"],
						"tu": ["05:00-09:00","16:00-20:00"],
						"we": ["05:00-09:00","16:00-20:00"],
						"th": ["05:00-09:00","16:00-20:00"],
						"fr": ["05:00-09:00","16:00-20:00"],
						"sa": [],
						"su": ["15:00-18:00"],
						"ph": []
					}
				}
			],
			"otherIntervalsByDays": [
				{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "05:00-09:00": 15, "16:00-20:00": 15, "20:00-22:00": 60 } },
				{ "days": [ "sa" ], "intervals": { "05:00-10:00": 60, "20:00-22:00": 60 } },
				{ "days": [ "su" ], "intervals": { "10:00-15:00": 60, "15:00-18:00": 15, "18:00-22:00": 60 } }
			],
			"allComputedIntervals": [
				{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "05:00-09:00": 15, "09:00-16:00": 30, "16:00-20:00": 15, "20:00-22:00": 60 } },
				{ "days": [ "sa" ], "intervals": { "05:00-10:00": 60, "10:00-20:00": 30, "20:00-22:00": 60 } },
				{ "days": [ "su" ], "intervals": { "07:00-10:00": 30, "10:00-15:00": 60, "15:00-18:00": 15, "18:00-22:00": 60 } }
			]
		};
		self.assertEqual(result, expected)

	def test_handles_over_midnight(self):
		tags = {
			"interval": "30",
			"interval:conditional": "15 @ (Mo-Fr 05:00-09:00,16:00-20:00; Su 15:00-18:00); 60 @ (Mo-Fr 20:00-22:00; Sa 05:00-10:00,20:00-22:00; Su 10:00-15:00,18:00-22:00); 90 @ (Mo-Fr 22:00-03:00)",
			"opening_hours": "Mo-Fr 05:00-03:00; Sa 05:00-22:00; Su 07:00-22:00; PH off"
		}
		result = Main().tagsToHoursObject(tags)
		expected = {
			"opens": {
				"mo": ["05:00-03:00"],
				"tu": ["05:00-03:00"],
				"we": ["05:00-03:00"],
				"th": ["05:00-03:00"],
				"fr": ["05:00-03:00"],
				"sa": ["05:00-22:00"],
				"su": ["07:00-22:00"],
				"ph": []
			},
			"defaultInterval": 30,
			"otherIntervals": [
				{
					"interval": 15,
					"applies": {
						"mo": ["05:00-09:00", "16:00-20:00"],
						"tu": ["05:00-09:00", "16:00-20:00"],
						"we": ["05:00-09:00", "16:00-20:00"],
						"th": ["05:00-09:00", "16:00-20:00"],
						"fr": ["05:00-09:00", "16:00-20:00"],
						"sa": [],
						"su": ["15:00-18:00"],
						"ph": []
					}
				},
				{
					"interval": 60,
					"applies": {
						"mo": ["20:00-22:00"],
						"tu": ["20:00-22:00"],
						"we": ["20:00-22:00"],
						"th": ["20:00-22:00"],
						"fr": ["20:00-22:00"],
						"sa": ["05:00-10:00", "20:00-22:00"],
						"su": ["10:00-15:00", "18:00-22:00"],
						"ph": []
					}
				},
				{
					"interval": 90,
					"applies": {
						"mo": ["22:00-03:00"],
						"tu": ["22:00-03:00"],
						"we": ["22:00-03:00"],
						"th": ["22:00-03:00"],
						"fr": ["22:00-03:00"],
						"sa": [],
						"su": [],
						"ph": []
					}
				}
			],
			"otherIntervalsByDays": [
				{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "05:00-09:00": 15, "16:00-20:00": 15, "20:00-22:00": 60, "22:00-03:00": 90 } },
				{ "days": [ "sa" ], "intervals": { "05:00-10:00": 60, "20:00-22:00": 60 } },
				{ "days": [ "su" ], "intervals": { "10:00-15:00": 60, "15:00-18:00": 15, "18:00-22:00": 60 } }
			],
			"allComputedIntervals": [
				{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "05:00-09:00": 15, "09:00-16:00": 30, "16:00-20:00": 15, "20:00-22:00": 60, "22:00-03:00": 90 } },
				{ "days": [ "sa" ], "intervals": { "05:00-10:00": 60, "10:00-20:00": 30, "20:00-22:00": 60 } },
				{ "days": [ "su" ], "intervals": { "07:00-10:00": 30, "10:00-15:00": 60, "15:00-18:00": 15, "18:00-22:00": 60 } }
			]
		}
		self.assertEqual(result, expected)

	def test_handles_over_midnight_2(self):
		tags = {
			"interval": "35",
			"interval:conditional": "60 @ (Mo-Su,PH 01:20-03:20)",
			"opening_hours": "Mo-Su,PH 21:50-06:55"
		}
		result = Main().tagsToHoursObject(tags)
		expected = {
			"opens": {
				"mo": ["21:50-06:55"],
				"tu": ["21:50-06:55"],
				"we": ["21:50-06:55"],
				"th": ["21:50-06:55"],
				"fr": ["21:50-06:55"],
				"sa": ["21:50-06:55"],
				"su": ["21:50-06:55"],
				"ph": ["21:50-06:55"]
			},
			"defaultInterval": 35,
			"otherIntervals": [
				{
					"interval": 60,
					"applies": {
						"mo": ["01:20-03:20"],
						"tu": ["01:20-03:20"],
						"we": ["01:20-03:20"],
						"th": ["01:20-03:20"],
						"fr": ["01:20-03:20"],
						"sa": ["01:20-03:20"],
						"su": ["01:20-03:20"],
						"ph": ["01:20-03:20"]
					}
				}
			],
			"otherIntervalsByDays": [
				{ "days": [ "mo", "tu", "we", "th", "fr", "sa", "su", "ph" ], "intervals": { "01:20-03:20": 60 } }
			],
			"allComputedIntervals": [
				{ "days": [ "mo", "tu", "we", "th", "fr", "sa", "su", "ph" ], "intervals": { "21:50-01:20": 35, "01:20-03:20": 60, "03:20-06:55": 35 } }
			]
		}
		self.assertEqual(result, expected)

	def test_works_opening_hours_having_PH_same_rest_week(self):
		tags = {
			"interval": "30",
			"interval:conditional": "15 @ (Mo-Su 10:00-13:00); 60 @ (PH 15:00-17:00)",
			"opening_hours": "05:00-22:00"
		}
		result = Main().tagsToHoursObject(tags)
		expected = {
			"opens": {
				"mo": ["05:00-22:00"],
				"tu": ["05:00-22:00"],
				"we": ["05:00-22:00"],
				"th": ["05:00-22:00"],
				"fr": ["05:00-22:00"],
				"sa": ["05:00-22:00"],
				"su": ["05:00-22:00"],
				"ph": ["05:00-22:00"]
			},
			"defaultInterval": 30,
			"otherIntervals": [
				{
					"interval": 15,
					"applies": {
						"mo": ["10:00-13:00"],
						"tu": ["10:00-13:00"],
						"we": ["10:00-13:00"],
						"th": ["10:00-13:00"],
						"fr": ["10:00-13:00"],
						"sa": ["10:00-13:00"],
						"su": ["10:00-13:00"],
						"ph": []
					}
				},
				{
					"interval": 60,
					"applies": {
						"mo": [],
						"tu": [],
						"we": [],
						"th": [],
						"fr": [],
						"sa": [],
						"su": [],
						"ph": ["15:00-17:00"]
					}
				}
			],
			"otherIntervalsByDays": [
				{ "days": [ "mo", "tu", "we", "th", "fr", "sa", "su" ], "intervals": { "10:00-13:00": 15 } },
				{ "days": [ "ph" ], "intervals": { "15:00-17:00": 60 } }
			],
			"allComputedIntervals": [
				{ "days": [ "mo", "tu", "we", "th", "fr", "sa", "su" ], "intervals": { "05:00-10:00": 30, "10:00-13:00": 15, "13:00-22:00": 30 } },
				{ "days": [ "ph" ], "intervals": { "05:00-15:00": 30, "15:00-17:00": 60, "17:00-22:00": 30 } }
			]
		}
		self.assertEqual(result, expected)

	#
	# intervalConditionalStringToObject
	#
	def test_intervalConditionalStringToObject_handles_standard_tag(self):
		intervalCond = "00:05 @ (Mo-Fr 07:00-10:00); 00:10 @ (Mo-Fr 16:30-19:00); 00:30 @ (Mo-Su 22:00-05:00)"
		result = Main().intervalConditionalStringToObject(intervalCond)
		expected = [
			{
				"interval": 5,
				"applies": {
					"mo": ["07:00-10:00"],
					"tu": ["07:00-10:00"],
					"we": ["07:00-10:00"],
					"th": ["07:00-10:00"],
					"fr": ["07:00-10:00"],
					"sa": [],
					"su": [],
					"ph": []
				}
			},
			{
				"interval": 10,
				"applies": {
					"mo": ["16:30-19:00"],
					"tu": ["16:30-19:00"],
					"we": ["16:30-19:00"],
					"th": ["16:30-19:00"],
					"fr": ["16:30-19:00"],
					"sa": [],
					"su": [],
					"ph": []
				}
			},
			{
				"interval": 30,
				"applies": {
					"mo": ["22:00-05:00"],
					"tu": ["22:00-05:00"],
					"we": ["22:00-05:00"],
					"th": ["22:00-05:00"],
					"fr": ["22:00-05:00"],
					"sa": ["22:00-05:00"],
					"su": ["22:00-05:00"],
					"ph": []
				}
			},
		]
		self.assertEqual(result, expected)

	#
	# _splitMultipleIntervalConditionalString
	#

	def test_splitMultipleIntervalConditionalString_handles_string_nosemicolons_in_oh_part(self):
		intervalCond = "00:05 @ (Mo-Fr 07:00-10:00); 00:05 @ (Mo-Fr 16:30-19:00); 00:30 @ (Mo-Su 22:00-05:00)"
		result = Main()._splitMultipleIntervalConditionalString(intervalCond)
		expected = ["00:05 @ (Mo-Fr 07:00-10:00)","00:05 @ (Mo-Fr 16:30-19:00)","00:30 @ (Mo-Su 22:00-05:00)"]
		self.assertEqual(result, expected)

	def test_splitMultipleIntervalConditionalString_handles_string_withsemicolons_in_ohpart(self):
		intervalCond = "00:05 @ (Mo-Fr 07:00-10:00 ; Su 16:30-19:00) ; 00:30 @ (Mo-Su 22:00-05:00)"
		result = Main()._splitMultipleIntervalConditionalString(intervalCond)
		expected = ["00:05 @ (Mo-Fr 07:00-10:00 ; Su 16:30-19:00)","00:30 @ (Mo-Su 22:00-05:00)"]
		self.assertEqual(result, expected)

	#
	# _readSingleIntervalConditionalString
	#

	def test_readSingleIntervalConditionalString_handles_basic(self):
		intervalCond = "00:10 @ (Sa-Su 06:00-22:00)"
		result = Main()._readSingleIntervalConditionalString(intervalCond)
		expected = {
			"interval": 10,
			"applies": {
				"mo": [],
				"tu": [],
				"we": [],
				"th": [],
				"fr": [],
				"sa": ["06:00-22:00"],
				"su": ["06:00-22:00"],
				"ph": []
			}
		}
		self.assertEqual(result, expected)

	def test_readSingleIntervalConditionalString_handles_oh_without_quotes(self):
		intervalCond = "15 @ Mo 06:00-22:00"
		result = Main()._readSingleIntervalConditionalString(intervalCond)
		expected = {
			"interval": 15,
			"applies": {
				"mo": ["06:00-22:00"],
				"tu": [],
				"we": [],
				"th": [],
				"fr": [],
				"sa": [],
				"su": [],
				"ph": []
			}
		}
		self.assertEqual(result, expected)

	def test_readSingleIntervalConditionalString_handles_nospaces(self):
		intervalCond = "15@Mo 06:00-22:00"
		result = Main()._readSingleIntervalConditionalString(intervalCond)
		expected = {
			"interval": 15,
			"applies": {
				"mo": ["06:00-22:00"],
				"tu": [],
				"we": [],
				"th": [],
				"fr": [],
				"sa": [],
				"su": [],
				"ph": []
			}
		}
		self.assertEqual(result, expected)

	#
	# _intervalConditionObjectToIntervalByDays
	#

	def test_intervalConditionObjectToIntervalByDays_works_distinct_days(self):
		intvObj = [
			{ "interval": 10, "applies": { "mo": [ "00:00-01:00" ], "tu": [ "01:00-02:00" ], "we": [ "02:00-03:00" ], "th": [], "fr": [], "sa": [], "su": [], "ph": [] } },
			{ "interval": 20, "applies": { "mo": [], "tu": [], "we": [], "th": [], "fr": [], "sa": [ "05:00-07:00" ], "su": [], "ph": [] } }
		]
		result = Main()._intervalConditionObjectToIntervalByDays(intvObj)
		expected = [
			{ "days": [ "mo" ], "intervals": { "00:00-01:00": 10 } },
			{ "days": [ "tu" ], "intervals": { "01:00-02:00": 10 } },
			{ "days": [ "we" ], "intervals": { "02:00-03:00": 10 } },
			{ "days": [ "sa" ], "intervals": { "05:00-07:00": 20 } }
		]
		self.assertEqual(result, expected)

	def test_intervalConditionObjectToIntervalByDays_merges_equal_days(self):
		intvObj = [
			{ "interval": 10, "applies": { "mo": [ "00:00-01:00" ], "tu": [ "00:00-01:00" ], "we": [ "00:00-01:00" ], "th": [], "fr": [], "sa": [], "su": [], "ph": [] } },
			{ "interval": 20, "applies": { "mo": [], "tu": [], "we": [], "th": [], "fr": [], "sa": [ "05:00-07:00" ], "su": [], "ph": [] } }
		]
		result = Main()._intervalConditionObjectToIntervalByDays(intvObj)
		expected = [
			{ "days": [ "mo", "tu", "we" ], "intervals": { "00:00-01:00": 10 } },
			{ "days": [ "sa" ], "intervals": { "05:00-07:00": 20 } }
		]
		self.assertEqual(result, expected)

	def test_intervalConditionObjectToIntervalByDays_distinguish_partially_equal_days(self):
		intvObj = [
			{ "interval": 10, "applies": { "mo": [ "00:00-01:00" ], "tu": [ "00:00-01:00", "05:00-07:00" ], "we": [ "00:00-01:00" ], "th": [], "fr": [], "sa": [], "su": [], "ph": [] } },
			{ "interval": 20, "applies": { "mo": [], "tu": [], "we": [], "th": [], "fr": [], "sa": [ "05:00-07:00" ], "su": [], "ph": [] } }
		]
		result = Main()._intervalConditionObjectToIntervalByDays(intvObj)
		expected = [
			{ "days": [ "mo", "we" ], "intervals": { "00:00-01:00": 10 } },
			{ "days": [ "tu" ], "intervals": { "00:00-01:00": 10, "05:00-07:00": 10 } },
			{ "days": [ "sa" ], "intervals": { "05:00-07:00": 20 } }
		]
		self.assertEqual(result, expected)

	#
	# _computeAllIntervals
	#

	def test_computeAllIntervals_works_all_data_set(self):
		interval = 10
		openingHours = {
			"mo": ["05:00-22:00"],
			"tu": ["05:00-22:00"],
			"we": ["05:00-22:00"],
			"th": ["05:00-22:00"],
			"fr": ["05:00-22:00"],
			"sa": ["07:00-23:00"],
			"su": [],
			"ph": []
		}
		intervalCondByDay = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "08:00-10:00": 5, "16:30-18:30": 5 } },
			{ "days": [ "sa" ], "intervals": { "11:00-13:00": 6 } }
		]
		result = Main()._computeAllIntervals(openingHours, interval, intervalCondByDay)

		expected = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "05:00-08:00": 10, "08:00-10:00": 5, "10:00-16:30": 10, "16:30-18:30": 5, "18:30-22:00": 10 } },
			{ "days": [ "sa" ], "intervals": { "07:00-11:00": 10, "11:00-13:00": 6, "13:00-23:00": 10 } }
		]

		self.assertEqual(result, expected)

	def test_computeAllIntervals_works_days_without_condinterval(self):
		interval = 10
		openingHours = {
			"mo": ["05:00-22:00"],
			"tu": ["05:00-22:00"],
			"we": ["05:00-22:00"],
			"th": ["05:00-22:00"],
			"fr": ["05:00-22:00"],
			"sa": ["07:00-23:00"],
			"su": [],
			"ph": []
		}
		intervalCondByDay = [
			{ "days": [ "mo", "fr" ], "intervals": { "08:00-10:00": 5, "16:30-18:30": 5 } },
			{ "days": [ "sa" ], "intervals": { "11:00-13:00": 6 } }
		]
		result = Main()._computeAllIntervals(openingHours, interval, intervalCondByDay)

		expected = [
			{ "days": [ "mo", "fr" ], "intervals": { "05:00-08:00": 10, "08:00-10:00": 5, "10:00-16:30": 10, "16:30-18:30": 5, "18:30-22:00": 10 } },
			{ "days": [ "tu", "we", "th" ], "intervals": { "05:00-22:00": 10 } },
			{ "days": [ "sa" ], "intervals": { "07:00-11:00": 10, "11:00-13:00": 6, "13:00-23:00": 10 } }
		]

		self.assertEqual(result, expected)

	def test_computeAllIntervals_works_sameoh_diffcondinterval(self):
		interval = 10
		openingHours = {
			"mo": ["05:00-22:00"],
			"tu": ["05:30-22:30"],
			"we": ["05:00-22:00"],
			"th": ["05:30-22:30"],
			"fr": ["05:00-22:00"],
			"sa": ["07:00-23:00"],
			"su": [],
			"ph": []
		}
		intervalCondByDay = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "08:00-10:00": 5, "16:30-18:30": 5 } },
			{ "days": [ "sa" ], "intervals": { "11:00-13:00": 6 } }
		]
		result = Main()._computeAllIntervals(openingHours, interval, intervalCondByDay)

		expected = [
			{ "days": [ "mo", "we", "fr" ], "intervals": { "05:00-08:00": 10, "08:00-10:00": 5, "10:00-16:30": 10, "16:30-18:30": 5, "18:30-22:00": 10 } },
			{ "days": [ "tu", "th" ], "intervals": { "05:30-08:00": 10, "08:00-10:00": 5, "10:00-16:30": 10, "16:30-18:30": 5, "18:30-22:30": 10 } },
			{ "days": [ "sa" ], "intervals": { "07:00-11:00": 10, "11:00-13:00": 6, "13:00-23:00": 10 } }
		]

		self.assertEqual(result, expected)

	def test_computeAllIntervals_works_nocondinterval(self):
		interval = 10
		openingHours = {
			"mo": ["05:00-22:00"],
			"tu": ["05:00-22:00"],
			"we": ["05:00-22:00"],
			"th": ["05:00-22:00"],
			"fr": ["05:00-22:00"],
			"sa": ["07:00-23:00"],
			"su": [],
			"ph": []
		}
		intervalCondByDay = "unset"
		result = Main()._computeAllIntervals(openingHours, interval, intervalCondByDay)

		expected = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "05:00-22:00": 10 } },
			{ "days": [ "sa" ], "intervals": { "07:00-23:00": 10 } }
		]

		self.assertEqual(result, expected)

	def test_computeAllIntervals_fail_if_novalid_condinterval(self):
		interval = 10
		openingHours = {
			"mo": ["05:00-22:00"],
			"tu": ["05:00-22:00"],
			"we": ["05:00-22:00"],
			"th": ["05:00-22:00"],
			"fr": ["05:00-22:00"],
			"sa": ["07:00-23:00"],
			"su": [],
			"ph": []
		}
		intervalCondByDay = "invalid"
		result = Main()._computeAllIntervals(openingHours, interval, intervalCondByDay)

		expected = "invalid"

		self.assertEqual(result, expected)

	def test_computeAllIntervals_works_if_oh_unset(self):
		interval = 10
		openingHours = "unset"
		intervalCondByDay = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "08:00-10:00": 5, "16:30-18:30": 5 } },
			{ "days": [ "sa" ], "intervals": { "11:00-13:00": 6 } }
		]
		result = Main()._computeAllIntervals(openingHours, interval, intervalCondByDay)

		expected = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "00:00-08:00": 10, "08:00-10:00": 5, "10:00-16:30": 10, "16:30-18:30": 5, "18:30-24:00": 10 } },
			{ "days": [ "sa" ], "intervals": { "00:00-11:00": 10, "11:00-13:00": 6, "13:00-24:00": 10 } },
			{ "days": [ "su", "ph" ], "intervals": { "00:00-24:00": 10 } }
		]

		self.assertEqual(result, expected)

	def test_computeAllIntervals_returns_only_condinterval_if_oh_invalid(self):
		interval = 10
		openingHours = "invalid"
		intervalCondByDay = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "08:00-10:00": 5, "16:30-18:30": 5 } },
			{ "days": [ "sa" ], "intervals": { "11:00-13:00": 6 } }
		]
		result = Main()._computeAllIntervals(openingHours, interval, intervalCondByDay)

		expected = intervalCondByDay

		self.assertEqual(result, expected)

	def test_computeAllIntervals_returns_only_condinterval_if_interval_unset(self):
		interval = "unset"
		openingHours = {
			"mo": ["05:00-22:00"],
			"tu": ["05:00-22:00"],
			"we": ["05:00-22:00"],
			"th": ["05:00-22:00"],
			"fr": ["05:00-22:00"],
			"sa": ["07:00-23:00"],
			"su": [],
			"ph": []
		}
		intervalCondByDay = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "08:00-10:00": 5, "16:30-18:30": 5 } },
			{ "days": [ "sa" ], "intervals": { "11:00-13:00": 6 } }
		]
		result = Main()._computeAllIntervals(openingHours, interval, intervalCondByDay)

		expected = intervalCondByDay

		self.assertEqual(result, expected)

	def test_computeAllIntervals_handles_oh_notcovering_condinterval(self):
		interval = 10
		openingHours = {
			"mo": ["05:00-12:00"],
			"tu": ["05:00-12:00"],
			"we": ["05:00-12:00"],
			"th": ["05:00-12:00"],
			"fr": ["05:00-12:00"],
			"sa": ["07:00-23:00"],
			"su": [],
			"ph": []
		}
		intervalCondByDay = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "08:00-10:00": 5, "16:30-18:30": 5 } },
			{ "days": [ "sa" ], "intervals": { "11:00-13:00": 6 } }
		]

		self.assertRaises(Exception, Main()._computeAllIntervals, openingHours, interval, intervalCondByDay)

	def test_computeAllIntervals_handles_condinterval_startend_overlaps_oh(self):
		interval = 30
		openingHours = {
			"mo": ["05:00-22:00"],
			"tu": ["05:00-22:00"],
			"we": ["05:00-22:00"],
			"th": ["05:00-22:00"],
			"fr": ["05:00-22:00"],
			"sa": ["05:00-22:00"],
			"su": [],
			"ph": []
		}
		intervalCondByDay = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "05:00-09:00": 15, "16:00-20:00": 15 } },
			{ "days": [ "sa" ], "intervals": { "05:00-10:00": 60, "20:00-22:00": 60 } }
		]
		result = Main()._computeAllIntervals(openingHours, interval, intervalCondByDay)

		expected = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "05:00-09:00": 15, "09:00-16:00": 30, "16:00-20:00": 15, "20:00-22:00": 30 } },
			{ "days": [ "sa" ], "intervals": { "05:00-10:00": 60, "10:00-20:00": 30, "20:00-22:00": 60 } }
		]

		self.assertEqual(result, expected)

	def test_computeAllIntervals_fixes_bug1(self):
		interval = 30
		openingHours = {
			"mo": ["05:00-22:00"],
			"tu": [],
			"we": [],
			"th": [],
			"fr": [],
			"sa": [],
			"su": [],
			"ph": []
		}
		intervalCondByDay = [
			{ "days": [ "mo" ], "intervals": { "05:00-09:00": 15, "16:00-20:00": 15, "20:00-22:00": 60 } }
		]
		result = Main()._computeAllIntervals(openingHours, interval, intervalCondByDay)

		expected = [
			{ "days": [ "mo" ], "intervals": { "05:00-09:00": 15, "09:00-16:00": 30, "16:00-20:00": 15, "20:00-22:00": 60 } }
		]

		self.assertEqual(result, expected)

	def test_computeAllIntervals_works_over_midnight(self):
		interval = 30
		openingHours = {
			"mo": ["05:00-03:00"],
			"tu": ["05:00-03:00"],
			"we": ["05:00-03:00"],
			"th": ["05:00-03:00"],
			"fr": ["05:00-03:00"],
			"sa": ["05:00-22:00"],
			"su": ["07:00-22:00"],
			"ph": []
		}
		intervalCondByDay = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "05:00-09:00": 15, "16:00-20:00": 15, "20:00-22:00": 60, "22:00-03:00": 90 } },
			{ "days": [ "sa" ], "intervals": { "05:00-10:00": 60, "20:00-22:00": 60 } },
			{ "days": [ "su" ], "intervals": { "10:00-15:00": 60, "15:00-18:00": 15, "18:00-22:00": 60 } }
		]
		result = Main()._computeAllIntervals(openingHours, interval, intervalCondByDay)

		expected = [
			{ "days": [ "mo", "tu", "we", "th", "fr" ], "intervals": { "05:00-09:00": 15, "09:00-16:00": 30, "16:00-20:00": 15, "20:00-22:00": 60, "22:00-03:00": 90 } },
			{ "days": [ "sa" ], "intervals": { "05:00-10:00": 60, "10:00-20:00": 30, "20:00-22:00": 60 } },
			{ "days": [ "su" ], "intervals": { "07:00-10:00": 30, "10:00-15:00": 60, "15:00-18:00": 15, "18:00-22:00": 60 } }
		]

		self.assertEqual(result, expected)

	def test_computeAllIntervals_works_over_midnight_2(self):
		interval = 30
		openingHours = {
			"mo": ["21:50-06:55"],
			"tu": ["21:50-06:55"],
			"we": ["21:50-06:55"],
			"th": ["21:50-06:55"],
			"fr": ["21:50-06:55"],
			"sa": ["21:50-06:55"],
			"su": ["21:50-06:55"],
			"ph": ["21:50-06:55"]
		}
		intervalCondByDay = [
			{ "days": [ "mo", "tu", "we", "th", "fr", "sa", "su", "ph" ], "intervals": { "01:20-03:20": 60 } }
		]
		result = Main()._computeAllIntervals(openingHours, interval, intervalCondByDay)

		expected = [
			{ "days": [ "mo", "tu", "we", "th", "fr", "sa", "su", "ph" ], "intervals": { "21:50-01:20": 30, "01:20-03:20": 60, "03:20-06:55": 30 } }
		]

		self.assertEqual(result, expected)

	#
	# _hourRangeOverlap
	#
	def test_overlap_is_False_if_disjoint(self):
		first = [ "07:00", "09:30" ]
		second = [ "16:30", "19:00" ]
		result = Main()._hourRangeOverlap(first, second)
		expected = False
		self.assertEqual(result, expected)

	def test_overlap_is_False_if_juxting(self):
		first = [ "05:00", "10:00" ]
		second = [ "10:00", "17:00" ]
		result = Main()._hourRangeOverlap(first, second)
		expected = False
		self.assertEqual(result, expected)

	def test_overlap_is_False_if_disjoint_and_over_midnight(self):
		first = [ "05:00", "10:00" ]
		second = [ "19:00", "02:00" ]
		result = Main()._hourRangeOverlap(first, second)
		expected = False
		self.assertEqual(result, expected)

	def test_overlap_is_True(self):
		first = [ "05:00", "17:00" ]
		second = [ "15:00", "17:00" ]
		result = Main()._hourRangeOverlap(first, second)
		expected = True
		self.assertEqual(result, expected)

	def test_overlap_is_True_with_over_midnight(self):
		first = [ "05:00", "17:00" ]
		second = [ "16:00", "02:00" ]
		result = Main()._hourRangeOverlap(first, second)
		expected = True
		self.assertEqual(result, expected)

	def test_overlap_is_True_with_both_over_midnight(self):
		first = [ "15:00", "02:00" ]
		second = [ "16:00", "03:00" ]
		result = Main()._hourRangeOverlap(first, second)
		expected = True
		self.assertEqual(result, expected)

	#
	# _hourRangeWithin
	#
	def test_is_True_if_smaller_inside_wider(self):
		wider = [ "05:00", "10:00" ]
		smaller = [ "06:00", "07:00" ]
		result = Main()._hourRangeWithin(wider, smaller)
		expected = True
		self.assertEqual(result, expected)

	def test_is_False_if_smaller_outside_wider(self):
		wider = [ "06:00", "07:00" ]
		smaller = [ "05:00", "10:00" ]
		result = Main()._hourRangeWithin(wider, smaller)
		expected = False
		self.assertEqual(result, expected)

	def test_is_True_if_smaller_equals_wider(self):
		wider = [ "06:00", "07:00" ]
		smaller = [ "06:00", "07:00" ]
		result = Main()._hourRangeWithin(wider, smaller)
		expected = True
		self.assertEqual(result, expected)

	def test_is_False_if_smaller_partly_outside_wider_1(self):
		wider = [ "06:00", "07:00" ]
		smaller = [ "06:30", "09:00" ]
		result = Main()._hourRangeWithin(wider, smaller)
		expected = False
		self.assertEqual(result, expected)

	def test_is_False_if_smaller_partly_outside_wider_2(self):
		wider = [ "06:00", "07:00" ]
		smaller = [ "05:30", "06:10" ]
		result = Main()._hourRangeWithin(wider, smaller)
		expected = False
		self.assertEqual(result, expected)

	def test_is_True_if_smaller_inside_wider_over_midnight_1(self):
		wider = [ "20:00", "02:00" ]
		smaller = [ "23:00", "01:00" ]
		result = Main()._hourRangeWithin(wider, smaller)
		expected = True
		self.assertEqual(result, expected)

	def test_is_True_if_smaller_inside_wider_over_midnight_2(self):
		wider = [ "20:00", "02:00" ]
		smaller = [ "23:00", "23:50" ]
		result = Main()._hourRangeWithin(wider, smaller)
		expected = True
		self.assertEqual(result, expected)

	def test_is_False_if_smaller_outside_wider_over_midnight(self):
		wider = [ "20:00", "02:00" ]
		smaller = [ "19:00", "03:00" ]
		result = Main()._hourRangeWithin(wider, smaller)
		expected = False
		self.assertEqual(result, expected)

	def test_is_True_if_smaller_equals_wider_over_midnight(self):
		wider = [ "20:00", "02:00" ]
		smaller = [ "20:00", "02:00" ]
		result = Main()._hourRangeWithin(wider, smaller)
		expected = True
		self.assertEqual(result, expected)

	def test_is_False_if_smaller_partly_outside_wider_1_over_midnight(self):
		wider = [ "20:00", "02:00" ]
		smaller = [ "20:30", "03:00" ]
		result = Main()._hourRangeWithin(wider, smaller)
		expected = False
		self.assertEqual(result, expected)

	def test_is_False_if_smaller_partly_outside_wider_2_over_midnight(self):
		wider = [ "20:00", "02:00" ]
		smaller = [ "19:00", "21:00" ]
		result = Main()._hourRangeWithin(wider, smaller)
		expected = False
		self.assertEqual(result, expected)

	def test_hourRangeWithin_fixes_bug_2(self):
		wider = [ "21:50", "06:55" ]
		smaller = [ "01:20", "03:20" ]
		result = Main()._hourRangeWithin(wider, smaller)
		expected = True
		self.assertEqual(result, expected)


	#
	# _mergeIntervalsSingleDay
	#
	def test_mergeIntervalsSingleDay_works_1hour_range(self):
		ohRanges = ["03:00-22:30"]
		interval = 10
		condIntervals = { "07:00-09:30": 5, "18:00-20:25": 4 }

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals)
		expected = { "03:00-07:00": 10, "07:00-09:30": 5, "09:30-18:00": 10, "18:00-20:25": 4, "20:25-22:30": 10 }

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_works_multiple_hour_ranges(self):
		ohRanges = ["03:00-12:00", "15:00-23:50"]
		interval = 10
		condIntervals = { "07:00-09:30": 5, "18:00-20:25": 4 }

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals)
		expected = { "03:00-07:00": 10, "07:00-09:30": 5, "09:30-12:00": 10, "15:00-18:00": 10, "18:00-20:25": 4, "20:25-23:50": 10 }

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_fails_oh_notcovering_condintervals(self):
		ohRanges = ["03:00-15:15"]
		interval = 10
		condIntervals = { "07:00-09:30": 5, "18:00-20:25": 4 }

		self.assertRaises(Exception, Main()._mergeIntervalsSingleDay, ohRanges, interval, condIntervals)

	def test_mergeIntervalsSingleDay_fails_multiple_hour_ranges_notcovering_condinterval(self):
		ohRanges = ["03:00-12:00", "15:00-20:00"]
		interval = 10
		condIntervals = { "01:00-04:00": 12, "11:30-12:30": 5, "14:00-16:00": 4, "19:00-22:00": 3 }

		self.assertRaises(Exception, Main()._mergeIntervalsSingleDay, ohRanges, interval, condIntervals)

	def test_mergeIntervalsSingleDay_works_oh_equals_condintervals(self):
		ohRanges = ["07:00-09:30", "18:00-20:25"]
		interval = 10
		condIntervals = { "07:00-09:30": 5, "18:00-20:25": 4 }

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals)
		expected = condIntervals

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_works_condint_overlapping_start_oh(self):
		ohRanges = ["05:00-22:00"]
		interval = 30
		condIntervals = { "05:00-09:00": 15, "16:00-20:00": 15 }

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals)
		expected = { "05:00-09:00": 15, "09:00-16:00": 30, "16:00-20:00": 15, "20:00-22:00": 30 }

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_works_condint_overlapping_end_oh(self):
		ohRanges = ["05:00-22:00"]
		interval = 30
		condIntervals = { "05:30-09:00": 15, "16:00-22:00": 15 }

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals)
		expected = { "05:00-05:30": 30, "05:30-09:00": 15, "09:00-16:00": 30, "16:00-22:00": 15 }

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_works_condint_overlaps_startend_oh(self):
		ohRanges = ["05:00-22:00"]
		interval = 30
		condIntervals = { "05:00-09:00": 15, "16:00-22:00": 15 }

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals)
		expected = { "05:00-09:00": 15, "09:00-16:00": 30, "16:00-22:00": 15 }

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_works_condint_overlaps_startend_oh2(self):
		ohRanges = ["05:00-22:00"]
		interval = 30
		condIntervals = { "05:00-09:00": 15 }

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals)
		expected = { "05:00-09:00": 15, "09:00-22:00": 30 }

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_works_with_conditional_intervals_going_over_midnight(self):
		ohRanges = ["17:00-03:00"];
		interval = 10;
		condIntervals = OrderedDict([("17:00-19:30", 15), ("23:30-03:00", 60)]);

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals)
		expected = { "17:00-19:30": 15, "19:30-23:30": 10, "23:30-03:00": 60 }

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_works_with_opening_hours_going_over_midnight_2(self):
		ohRanges = ["17:00-03:00"];
		interval = 10;
		condIntervals = { "17:00-19:30": 15 };

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals)
		expected = { "17:00-19:30": 15, "19:30-03:00": 10 }

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_works_with_opening_hours_going_over_midnight_3(self):
		ohRanges = ["17:00-03:00"];
		interval = 10;
		condIntervals = { "17:00-02:00": 15 };

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals)
		expected = { "17:00-02:00": 15, "02:00-03:00": 10 }

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_fails_with_conditional_intervals_not_exclusive(self):
		ohRanges = ["03:00-20:00"];
		interval = 10;
		condIntervals = { "03:00-10:00": 15, "09:30-15:00": 30, "17:00-20:00": 13 };

		self.assertRaises(Exception, Main()._mergeIntervalsSingleDay, ohRanges, interval, condIntervals)

	def test_mergeIntervalsSingleDay_fails_with_conditional_intervals_not_exclusive_over_midnight(self):
		ohRanges = ["17:00-03:00"];
		interval = 10;
		condIntervals = { "17:00-19:30": 15, "19:30-01:00": 15, "23:30-03:00": 60 };

		self.assertRaises(Exception, Main()._mergeIntervalsSingleDay, ohRanges, interval, condIntervals)

	def test_mergeIntervalsSingleDay_fixes_bug_1(self):
		ohRanges = ["05:00-22:00"];
		interval = 30;
		condIntervals = { "05:00-09:00": 15, "16:00-20:00": 15, "20:00-22:00": 60 };

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals);

		expected = { "05:00-09:00": 15, "09:00-16:00": 30, "16:00-20:00": 15, "20:00-22:00": 60 };

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_fixes_bug_2_0(self):
		ohRanges = ["21:50-06:55"];
		interval = 30;
		condIntervals = { "01:20-03:20": 60 };

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals);

		expected = { "21:50-01:20": 30, "01:20-03:20": 60, "03:20-06:55": 30 };

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_fixes_bug_2_1(self):
		ohRanges = ["21:50-06:55"];
		interval = 30;
		condIntervals = { "22:00-23:00": 45, "01:20-03:20": 60 };

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals);

		expected = { "21:50-22:00": 30, "22:00-23:00": 45, "23:00-01:20": 30, "01:20-03:20": 60, "03:20-06:55": 30 };

		self.assertEqual(result, expected)

	def test_mergeIntervalsSingleDay_fixes_bug_2_2(self):
		ohRanges = ["07:00-22:00"];
		interval = 30;
		condIntervals = { "15:00-18:00": 15, "10:00-15:00": 60, "18:00-22:00": 60 };

		result = Main()._mergeIntervalsSingleDay(ohRanges, interval, condIntervals);

		expected = { "07:00-10:00": 30, "10:00-15:00": 60, "15:00-18:00": 15, "18:00-22:00": 60 };

		self.assertEqual(result, expected)

	#
	# intervalStringToMinutes
	#
	def test_intervalStringToMinutes_hhmmss1(self):
		interval = "01:00:00"
		result = Main().intervalStringToMinutes(interval)
		expected = 60
		self.assertEqual(result, expected)

	def test_intervalStringToMinutes_hhmmss2(self):
		interval = "01:30:00"
		result = Main().intervalStringToMinutes(interval)
		expected = 90
		self.assertEqual(result, expected)


	def test_intervalStringToMinutes_hhmmss3(self):
		interval = "02:45:30"
		result = Main().intervalStringToMinutes(interval)
		expected = 165.5
		self.assertEqual(result, expected)


	def test_intervalStringToMinutes_hhmm1(self):
		interval = "01:00"
		result = Main().intervalStringToMinutes(interval)
		expected = 60
		self.assertEqual(result, expected)


	def test_intervalStringToMinutes_hhmm2(self):
		interval = "03:12"
		result = Main().intervalStringToMinutes(interval)
		expected = 192
		self.assertEqual(result, expected)


	def test_intervalStringToMinutes_mm1(self):
		interval = "15"
		result = Main().intervalStringToMinutes(interval)
		expected = 15
		self.assertEqual(result, expected)


	def test_intervalStringToMinutes_mm2(self):
		interval = "135"
		result = Main().intervalStringToMinutes(interval)
		expected = 135
		self.assertEqual(result, expected)


	def test_intervalStringToMinutes_hmm(self):
		interval = "3:10"
		result = Main().intervalStringToMinutes(interval)
		expected = 190
		self.assertEqual(result, expected)


	def test_intervalStringToMinutes_invalid(self):
		interval = "12 minutes"
		self.assertRaises(Exception, Main().intervalStringToMinutes, interval)


if __name__ == '__main__':
	unittest.main()



# -*- coding: utf-8 -*-

import re
import functools
from .openinghoursparser import OpeningHoursParser

TAG_UNSET = "unset"
TAG_INVALID = "invalid"
DAYS_ID = [ "mo", "tu", "we", "th", "fr", "sa", "su", "ph" ]

class Main:
	"""
	Main class of the library.
	It contains all main functions which can help managing public transport hours.
	"""

	def tagsToGtfs(self, tags):
		"""
		Convert OpenStreetMap tags into a GTFS-like format (list of dict having format { eachWeekDay: True/False, start_time: string, end_time: string, headway: int }.
		Parsed tags are : interval=\*, opening_hours=\* and interval:conditional=\*

		# param tags (dict): OpenStreetMap tags
		# return dict[]: list of dictionaries, each one representing a line of GTFS hours CSV file
		"""
		hours = self.tagsToHoursObject(tags)

		if hours['allComputedIntervals'] == TAG_INVALID:
			raise Exception("OSM tags describing route hours are probably invalid, and can't be read")
		elif hours['allComputedIntervals'] == TAG_UNSET:
			return []
		else:
			result = []

			daysId = [ "mo", "tu", "we", "th", "fr", "sa", "su", "ph" ]

			# Read each period(days, intervals)
			for period in sorted(hours['allComputedIntervals'], key=lambda x: daysId.index(x['days'][0])):
				periodGtfs = {}

				# Interpret days (we ignore ph as not used by GTFS)
				days = [ d for d in period['days'] if d != "ph" ]
				if len(days) > 0:
					periodGtfs['monday'] = "mo" in days
					periodGtfs['tuesday'] = "tu" in days
					periodGtfs['wednesday'] = "we" in days
					periodGtfs['thursday'] = "th" in days
					periodGtfs['friday'] = "fr" in days
					periodGtfs['saturday'] = "sa" in days
					periodGtfs['sunday'] = "su" in days

					# Interpret intervals
					for hourRange in sorted(period['intervals'].keys()):
						periodHourGtfs = dict(periodGtfs)
						periodHourGtfs['start_time'] = hourRange.split("-")[0] + ":00"
						periodHourGtfs['end_time'] = hourRange.split("-")[1] + ":00"
						periodHourGtfs['headway'] = period['intervals'][hourRange] * 60

						# Add to result
						result.append(periodHourGtfs)

			return result

	def tagsToHoursObject(self, tags):
		"""
		Converts OpenStreetMap tags into a ready-to-use object representing the hours of the public transport line.
		Parsed tags are : interval=\*, opening_hours=\* and interval:conditional=\*

		# param tags (dict): The list of tags from OpenStreetMap
		# return dict: The hours of the line, with structure { opens: object in format given by #OpeningHoursParser.gettable(), defaultInterval: minutes (int), otherIntervals: interval rules object, otherIntervalsByDays: list of interval by days (structure: { days: string[], intervals: { hoursRange: interval } }), allComputedIntervals: same as otherIntervalsByDays but taking also default interval and opening_hours }. Each field can also have value "unset" if no tag is defined, or "invalid" if tag can't be read.
		"""

		# Read opening_hours
		opens = TAG_UNSET
		try:
			opens = OpeningHoursParser(tags['opening_hours']).getTable() if 'opening_hours' in tags else TAG_UNSET
		except:
			opens = TAG_INVALID

		# Read interval
		interval = TAG_UNSET
		try:
			interval = self.intervalStringToMinutes(tags['interval']) if 'interval' in tags else TAG_UNSET
		except:
			interval = TAG_INVALID

		# Read interval:conditional
		intervalCond = TAG_UNSET
		intervalCondByDay = TAG_UNSET
		try:
			intervalCond = self.intervalConditionalStringToObject(tags["interval:conditional"]) if "interval:conditional" in tags else TAG_UNSET
			intervalCondByDay = self._intervalConditionObjectToIntervalByDays(intervalCond) if intervalCond != TAG_UNSET else TAG_UNSET
		except:
			intervalCond = TAG_INVALID
			intervalCondByDay = TAG_INVALID

		# Create computed calendar of intervals using previous data
		computedIntervals = TAG_UNSET
		try:
			computedIntervals = self._computeAllIntervals(opens, interval, intervalCondByDay)
		except:
			computedIntervals = TAG_INVALID

		# Send result
		return {
			"opens": opens,
			"defaultInterval": interval,
			"otherIntervals": intervalCond,
			"otherIntervalsByDays": intervalCondByDay,
			"allComputedIntervals": computedIntervals
		}

	def intervalConditionalStringToObject(self, intervalConditional):
		"""
		Reads an interval:conditional=* tag from OpenStreetMap, and converts it into a dict.

		# param intervalConditional (string): The {@link https://wiki.openstreetmap.org/wiki/Key:interval|interval:conditional} tag
		# return (dict[]): A list of rules, each having structure { interval: minutes (int), applies: {@link #gettable|opening hours table} }
		"""
		return [ self._readSingleIntervalConditionalString(p) for p in self._splitMultipleIntervalConditionalString(intervalConditional) ]

	def _splitMultipleIntervalConditionalString(self, intervalConditional):
		"""
		Splits several conditional interval rules being separated by semicolon.

		# param intervalConditional (string)
		# return (string[]): List of single rules
		"""
		if "(" in intervalConditional:
			semicolons = [i for i, ltr in enumerate(intervalConditional) if ltr == ";"]
			cursor = 0
			stack = []

			while len(semicolons) > 0:
				scid = semicolons[0]
				part = intervalConditional[cursor:scid]

				if re.search("^[^\(\)]+$", part) or re.search("\(.*\)", part):
					stack.append(part)
					cursor = scid+1

				semicolons.pop(0)

			stack.append(intervalConditional[cursor:])
			return [ p.strip() for p in stack if len(p.strip()) > 0 ]
		else:
			return [ p.strip() for p in intervalConditional.split(";") if len(p.strip()) > 0 ]

	def _readSingleIntervalConditionalString(self, intervalConditional):
		"""
		Parses a single conditional interval value (for example : `15 @ (08:00-15:00)`).
		This should be used as many times as you have different rules (separated by semicolon).

		# param intervalConditional (string)
		# return (dict): dictionary with structure { interval: minutes (int), applies: OpeningHoursParser.gettable() structure} }
		"""
		result = {}
		parts = [ p.strip() for p in intervalConditional.split("@") ]

		if len(parts) != 2:
			raise Exception("Conditional interval can't be parsed : "+intervalConditional)

		# Read interval
		result['interval'] = self.intervalStringToMinutes(parts[0])

		# Read opening hours
		if re.search("^\(.*\)$", parts[1]):
			parts[1] = parts[1][1:len(parts[1])-1]

		result['applies'] = OpeningHoursParser(parts[1]).getTable()

		return result

	def _intervalConditionObjectToIntervalByDays(self, intervalConditionalObject):
		"""
		Transforms an object containing the conditional intervals into an object structured day by day.
		"""
		result = []
		itvByDay = {}

		# List hours -> interval day by day
		for itv in intervalConditionalObject:
			for day, hours in itv['applies'].items():
				if day not in itvByDay:
					itvByDay[day] = {}
				for h in hours:
					itvByDay[day][h] = itv['interval']

		# Merge days
		for day, intervals in itvByDay.items():
			if len(intervals) > 0:
				# Look for identical days
				ident = [ r for r in result if r['intervals'] == intervals ]

				if len(ident) == 1:
					ident[0]['days'].append(day)
				else:
					result.append({ "days": [ day ], "intervals": intervals })

		# Sort by days
		for itv in result:
			itv['days'].sort(key = lambda x: DAYS_ID.index(x))

		result.sort(key = lambda x: DAYS_ID.index(x['days'][0]))

		return result

	def _flatList(self, myList):
		flatList=[]
		for eachList in myList:
			for eachItem in eachList:
				flatList.append(eachItem)
		return flatList

	def _computeAllIntervals(self, openingHours, interval, intervalCondByDay):
		"""
		Reads all information, and generates a merged calendar of all intervals.
		"""

		# If opening hours or interval is invalid, returns interval conditional as is
		if openingHours == TAG_INVALID or interval == TAG_INVALID or interval == TAG_UNSET or intervalCondByDay == TAG_INVALID:
			return TAG_INVALID if (openingHours == TAG_INVALID or interval == TAG_INVALID) and intervalCondByDay == TAG_UNSET else intervalCondByDay
		else:
			myIntervalCondByDay = [] if intervalCondByDay == TAG_UNSET else intervalCondByDay

			# Check opening hours, if missing we default to 24/7
			myOH = openingHours
			if openingHours == TAG_UNSET:
				myOH = OpeningHoursParser("24/7").getTable()

			# Copy existing intervals (split day by day)
			result = []
			for di in myIntervalCondByDay:
				for d in di['days']:
					result.append({ "days": [d], "intervals": di['intervals'] })

			# Complete existing days
			result = list(result)
			for di in result:
				di['intervals'] = self._mergeIntervalsSingleDay(myOH[di['days'][0]], interval, di['intervals'])

			# List days not in myIntervalCondByDay, and add directly opening hours
			# Was: daysInCondInt = [...new Set(myIntervalCondByDay.map(d => d.days).flat())]
			daysInCondInt = list(set(self._flatList([ d['days'] for d in myIntervalCondByDay ])))
			missingDays = [ d for d in myOH if d not in daysInCondInt ]
			missingDaysOH = {}
			for day in missingDays:
				missingDaysOH[day] = myOH[day]

			result = result + self._intervalConditionObjectToIntervalByDays([{ "interval": interval, "applies": missingDaysOH }])

			# Merge similar days
			i = 1
			while i < len(result):
				j = 0
				while j < i:
					if result[i]['intervals'] == result[j]['intervals']:
						result[j]['days'] = result[j]['days'] + result[i]['days']
						del result[i]
						i -= 1
						break
					j += 1
				i += 1

			# Sort results by day
			for r in result:
				r['days'].sort(key=lambda x: DAYS_ID.index(x))

			result.sort(key=lambda x: DAYS_ID.index(x['days'][0]))

			return result

	def _hourRangeWithin(self, wider, smaller):
		"""
		Check if an hour range is contained in another one
		"""
		if wider == smaller:
			return True
		else:
			# During day
			if wider[0] <= wider[1]:
				if smaller[0] > smaller[1]:
					return False
				else:
					return wider[0] <= smaller[0] and smaller[0] < wider[1] and wider[0] < smaller[1] and smaller[1] <= wider[1]
			# Over midnight
			else:
				# Either before or after midnight
				if smaller[0] <= smaller[1]:
					# All after wider start
					if wider[0] <= smaller[0] and wider[0] <= smaller[1]:
						return True
					# All before wider end
					elif "00:00" <= smaller[0] and smaller[1] <= wider[1]:
						return True
					else:
						return False
				# Over midnight
				else:
					return wider[0] <= smaller[0] and smaller[0] <= "24:00" and "00:00" <= smaller[1] and smaller[1] <= wider[1]

	def _hourRangeOverlap(self, first_range, second_range):
		"""
		Check if an hour range overlap another one
		"""
		if first_range == second_range:
			return False
		else:
			if not first_range[0] <= second_range[0]:
				_ = first_range
				first_range = second_range
				second_range = _

			# First one is during day
			if first_range[0] <= first_range[1]:
				return second_range[0] < first_range[1]

			# First one is over midnight
			else:
				# Second one is during day (Either before or after midnight)
				if second_range[0] <= second_range[1]:
					if first_range[0] <= second_range[0] and first_range[0] <= second_range[1]:
						return True
					else:
						return False
				# Second one is over midnight
				else:
					return True

	def _mergeIntervalsSingleDay(self, hours, interval, condIntervals):
		"""
		Add default interval within opening hours to conditional intervals
		"""
		hourRangeToArr = lambda hr: [ h.split("-") for h in hr ]
		ohHours = hourRangeToArr(hours)
		condHours = hourRangeToArr(condIntervals)

		# Check all conditional hours belong into opening hours
		for condHours_elem in condHours:
				if not any([ohh for ohh in ohHours if self._hourRangeWithin(ohh, condHours_elem) ]):
						raise Exception("Conditional intervals are not contained in opening hours")

		# Check conditional hours are not overlapping
		goneOverMidnight = False
		sortedCondHours = sorted(list(condHours), cmp=lambda x,y: self.intervalStringToMinutes(x[0])-self.intervalStringToMinutes(y[0]))

		for i in range(len(sortedCondHours)):
			ch = sortedCondHours[i]
			if not goneOverMidnight:
				if ch[0] > ch[1]:
					goneOverMidnight = True

				for j in range(i):
					if self._hourRangeOverlap(ch, sortedCondHours[j]):
						raise Exception("Conditional intervals are not exclusive (they overlaps)")
			else:
				raise Exception("Conditional intervals are not exclusive (several intervals after midnight)")

		ohHoursWithoutConds = []

		for ohh in ohHours:
			holes = []
			ohhOverMidnight = ohh[0] > ohh[1]
			thisCondHours = [ ch for ch in (condHours if ohhOverMidnight else sortedCondHours) if self._hourRangeWithin(ohh, ch) ]

			for i in range(len(thisCondHours)):
				ch = thisCondHours[i]
				isFirst = i == 0
				isLast = i == len(thisCondHours) - 1

				if isFirst and ohh[0] < ch[0]:
					holes.append(ohh[0])
					holes.append(ch[0])

				if not isFirst:
					if thisCondHours[i-1][1] < ch[0] or (ohhOverMidnight and thisCondHours[i-1][1] > ch[0]):
						holes.append(thisCondHours[i-1][1])
						holes.append(ch[0])

				if isLast:
					appendLast = False

					# opening hours before midnight
					if ohh[0] < ohh[1]:
						if ch[1] < ohh[1]:
							appendLast = True
					# opening hours going after midnight
					else:
						# current range before midnight
						if ch[0] < ch[1] and ohh[0] <= ch[0] and ch[1] <= "24:00":
							appendLast = True
						# current range after midnight
						elif ch[0] < ch[1] and "00:00" <= ch[0] and ch[1] <= ohh[1]:
							if isFirst:
								holes.append(ohh[0])
								holes.append(ch[0])
							appendLast = True
						# current range going through midnight
						else:
							# Current range ending before opening hour end
							if ch[1] < ohh[1]:
								appendLast = True

					if appendLast:
						holes.append(ch[1])
						holes.append(ohh[1])

			ohHoursWithoutConds += [ holes[i-1]+"-"+holes[i] for i in range(len(holes)) if i % 2 == 1 ]

		result = {}
		for h in ohHoursWithoutConds:
			result[h] = interval

		result.update(condIntervals)

		return result

	def intervalStringToMinutes(self, interval):
		"""
		Converts an interval=* string into an amount of minutes

		>>> intervalStringToMinutes("00:10")
		10
		"""
		interval = interval.strip()

		# hh:mm:ss
		if re.search("^\d{1,2}:\d{2}:\d{2}$", interval):
			parts = [ int(t) for t in interval.split(":") ]
			return parts[0] * 60 + parts[1] + parts[2] / 60.0

		# hh:mm
		elif re.search("^\d{1,2}:\d{2}$", interval):
			parts = [ int(t) for t in interval.split(":") ]
			return parts[0] * 60 + parts[1]

		# mm
		elif re.search("^\d+$", interval):
			return int(interval)

		# invalid
		else:
			raise Exception("Interval value can't be parsed : "+interval)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


with open('README.md') as f:
    readme = f.read()

setup(
    name='transporthours',
    version='0.0.3',
    description='Package for handling public transport routes opening hours from OpenStreetMap',
    long_description_content_type='text/markdown',
    long_description=readme,
    author='Adrien Pavie & Jungle Bus',
    author_email='panieravide@riseup.net',
    url='https://github.com/Jungle-Bus/transport-hours-py',
    license='GNU LGPLv3',
    packages=find_packages(exclude=('tests')),
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Topic :: Scientific/Engineering :: GIS",
        "Topic :: Software Development :: Libraries"
    ],
    python_requires='>=2.7'
)
